<?php 

/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function node_gallery_api_views_data() {
  $data['node_gallery_relationship']['table']['group'] = t("Node Gallery: Item");
  $data['node_gallery_relationship']['table']['join'] = array(
    'node' => array(
      'left_field' => 'nid',
      'field' => 'nid',
    ),
  );
  $data['node_gallery_relationship']['ngid'] = array(
    'title' => t("Parent Gallery"),
    'help' => t("The ID of the item's gallery."),
    'field' => array(
      'handler' => 'views_handler_field_node',
      'click sortable' => TRUE,
    ),
    'relationship' => array(
      'label' => t('Parent Gallery'),
      'base' => 'node',
      'base field' => 'nid',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_node_nid',
      'name field' => 'title', // the field to display in the summary.
      'numeric' => TRUE,
      'validate type' => 'nid',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );
  $data['node_gallery_relationship']['weight'] = array(
    'title' => t('Item weight'),
    'help' => t('The weight assigned to the item. Used for sorting within a gallery.'),
    'field' => array(
      'handler' => 'views_handler_field',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );
  $data['node_gallery_relationship']['navigator'] = array(
    'title' => t('Item Navigator'),
    'help' => t('A themed navigator for browsing previous and next images.'),
    'field' => array(
      'handler' => 'node_gallery_api_views_handler_item_navigator',
      'click sortable' => FALSE,
      'notafield' => TRUE,
    ),
  );

  $data['node_gallery_galleries']['table']['group'] = t('Node Gallery: Gallery');
  $data['node_gallery_galleries']['table']['join'] = array(
    'node' => array(
      'left_field' => 'nid',
      'field' => 'ngid',
    ),
  );
  $node_schema = drupal_get_schema('node');
  $fields = array_keys($node_schema['fields']);
  $pos = array_search('nid', $fields);
  if ($pos !== FALSE) {
    array_splice($fields, $pos, 1);
  }
  $node_fields = 'node.' . implode(', node.', $fields);

  $data['node_gallery_galleries']['newest_item'] = array(
    'title' => t('Gallery Newest Item'),
    'help' => t('Relationship to link the newest item for the gallery.'),
    'relationship' => array(
      'label' => t('Gallery Newest Item'),
      'base' => 'node',
      'table formula' => '(SELECT ngid, ' . $node_fields . ', MAX(ngi.nid) as nid FROM {node_gallery_relationship} ngi JOIN {node} node ON ngi.nid = node.nid GROUP BY ngid)',
      'base field' => 'ngid',
      'relationship field' => 'ngid',
    ),
  );
  $data['node_gallery_galleries']['oldest_item'] = array(
    'title' => t('Gallery Oldest Item'),
    'help' => t('Relationship to link the oldest item for the gallery.'),
    'relationship' => array(
      'label' => t('Gallery Oldest Item'),
      'base' => 'node',
      'table formula' => '(SELECT ngid, ' . $node_fields . ', MIN(ngi.nid) AS nid, node.* FROM {node_gallery_relationship} ngi JOIN {node} node ON ngi.nid = node.nid GROUP BY ngid)',
      'base field' => 'ngid',
      'relationship field' => 'ngid',
    ),
  );
  $data['node_gallery_galleries']['cover_item'] = array(
    'title' => t('Gallery Cover Item'),
    'help' => t('Relationship to link the cover item for the gallery.'),
    'relationship' => array(
      'label' => t('Gallery Cover Item'),
      'base' => 'node',
      'base field' => 'nid',
    ),
  );
  $data['node_gallery_galleries']['pub_item_count'] = array(
    'title' => t('Published Item Count'),
    'help' => t('The number of published items in the gallery.'),
    'real field' => 'pub_item_count',
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'numeric' => TRUE,
      'validate type' => 'numeric',
    ),
  );
  $data['node_gallery_galleries']['item_count'] = array(
    'title' => t('Item Count'),
    'help' => t('The total number of published and unpublished items in the gallery.'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'numeric' => TRUE,
      'validate type' => 'numeric',
    ),
  );
  $data['node_gallery_galleries']['comments_count'] = array(
    'title' => t('Total comments'),
    'help' => t('The total number of comments on the gallery and all its images.'),
    'field' => array(
      'handler' => 'node_gallery_api_views_handler_comments_count',
      'click sortable' => TRUE,
      'notafield' => TRUE,
    ),
  );
  $data['node_gallery_galleries']['item_date'] = array(
    'title' => t('Item Updated Date'),
    'help' => t('The last changed date of an item in the gallery.'),
    'field' => array(
      'handler' => 'node_gallery_views_handler_item_date',
      'click sortable' => TRUE,
      'notafield' => TRUE,
    ),
  );
  $data['node_gallery_galleries']['ngid'] = array(
    'title' => t("Gallery ID"),
    'help' => t("The ID of the gallery."),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
  );
  return $data;
}

/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function node_gallery_api_views_handlers() {
  return array(
    'info' => array(
      'path' => drupal_get_path('module', 'node_gallery_api') . '/includes/views',
    ),
    'handlers' => array(
      'node_gallery_api_views_handler_comments_count' => array(
        'parent' => 'views_handler_field_numeric',
      ),
      'node_gallery_api_views_handler_item_date' => array(
        'parent' => 'views_handler_field_date',
      ),
      'node_gallery_api_views_handler_item_navigator' => array(
        'parent' => 'views_handler_field',
      ),
    ),
  );
}