<?php

/**
 * OG behavior handler.
 */
class NodeGalleryBehaviorHandler extends EntityReference_BehaviorHandler_Abstract {

  public function access($field, $instance) {
    return $field['settings']['handler'] == 'node_gallery';
  }

  public function load($entity_type, $entities, $field, $instances, $langcode, &$items) {
    // Get the Node galleries.
    foreach ($entities as $entity) {
      $relationships = node_gallery_api_get_relationships(NULL, $entity->nid);
      foreach ($relationships as $relationship) {
        $items[$entity->nid][] = array(
          'target_id' => $relationship->ngid,
        );
      }
    }
  }

  public function insert($entity_type, $entity, $field, $instance, $langcode, &$items) {
    $this->NodeGalleryRelationshipCrud($entity_type, $entity, $field, $instance, $langcode, $items);
    
    // TODO: Clear the items array so that entityreference doesn't double-store the information.
    // This actually works if you uncomment the line below. However, it does not work correctly 
    // once you install FileField Paths. Not sure why, but it was not easy to work around. Loads 
    // of spaghetti code. Much cleaner to just double-store the data until this problem can be 
    // addressed properly. Want to fix this? I'd love you to. Please send me a patch.
    //$items = array();
  }

  public function update($entity_type, $entity, $field, $instance, $langcode, &$items) {
    $this->NodeGalleryRelationshipCrud($entity_type, $entity, $field, $instance, $langcode, $items);
    
    // TODO: Clear the items array so that entityreference doesn't double-store the information.
    // See comment in insert().
    //$items = array();
  }

  /**
   * Implements EntityReference_BehaviorHandler_Abstract::Delete()
   *
   * CRUD memberships from field, or if entity is marked for deleteing,
   * delete all the OG membership related to it.
   *
   * @see og_entity_delete().
   */
  public function delete($entity_type, $entity, $field, $instance, $langcode, &$items) {
    // Delete all memberships related to this item node.
    $items = array();
    $this->NodeGalleryRelationshipCrud($entity_type, $entity, $field, $instance, $langcode, $items);
  }

  /**
   * Create, update or delete OG membership based on field values.
   */
  public function NodeGalleryRelationshipCrud($entity_type, $entity, $field, $instance, $langcode, &$items) {
    $diff = $this->galleriesGetDiff($entity_type, $entity, $field, $instance, $langcode, $items);
    if (!$diff) {
      return;
    }

    $field_name = $field['field_name'];
    $group_type = $field['settings']['target_type'];

    $diff += array('insert' => array(), 'delete' => array());
    
    // Delete first, so we don't trigger cardinality errors.
    if ($diff['delete']) {
      entity_delete_multiple('node_gallery_relationship', array_keys($diff['delete']));
      foreach(array_values($diff['delete']) as $ngid) {
        if(node_gallery_api_get_cover_nid($ngid) == $entity->nid) {
          node_gallery_api_reset_cover_item($ngid);
        }
        node_gallery_api_clear_gallery_caches($ngid);
        node_gallery_api_update_image_counts($ngid);
      }
    }
    
    if(!empty($diff['insert'])) {
      $relationship_type = node_gallery_api_get_relationship_type(NULL, $entity->type);
      foreach ($diff['insert'] as $ngid) {
        $r = new NodeGalleryRelationship();
        $r->relationship_type = $relationship_type->id;
        $r->nid = $entity->nid;
        $r->ngid = $ngid;
        $r->weight = NODE_GALLERY_DEFAULT_WEIGHT;
        $r->save();
      }
    }
  }

  /**
   * Get the difference in group audience for a saved field.
   *
   * @return
   *   Array with all the differences, or an empty array if none found.
   */
  public function galleriesGetDiff($entity_type, $entity, $field, $instance, $langcode, $items) {
    $return = FALSE;

    $current_relationships = node_gallery_api_get_relationships(NULL, $entity->nid);
    

    $new_relationships = array();
    foreach ($items as $item) {
      $new_relationships[$item['target_id']] = TRUE;
    }

    foreach ($current_relationships as $current_relationship) {
      $ngid = $current_relationship->ngid;
      if (empty($new_relationships[$ngid])) {
        // Relationship was deleted.
        $return['delete'][$current_relationship->id] = $current_relationship->ngid;
        unset($new_relationships[$ngid]);
      }
      else {
        // Existing membership.
        unset($new_relationships[$ngid]);
      }
    }
    if ($new_relationships) {
      // New memberships.
      $return['insert'] = array_keys($new_relationships);
    }

    return $return;
  }
}
