<?php

/**
 * Node Gallery API selection handler.
 */
class NodeGallerySelectionHandler extends EntityReference_SelectionHandler_Generic_node {

/**
   * Implements EntityReferenceHandler::getInstance().
   */
  public static function getInstance($field, $instance = NULL, $entity_type = NULL, $entity = NULL) {
    return new NodeGallerySelectionHandler($field, $instance, $entity_type, $entity);
  }

/**
   * Override settings form().
   */
  public static function settingsForm($field, $instance) {
    $form = parent::settingsForm($field, $instance);
    $entity_type = $field['settings']['target_type'];
    $entity_info = entity_get_info($entity_type);
    $relationship_types = node_gallery_api_get_relationship_type();
    $rt_options = array();
    foreach ($relationship_types as $rt) {
      $rt_options[$rt->id] = $rt->label;
    }
    $form['node_gallery_relationship_type'] = array(
      '#title' => t('Relationship Type'),
      '#type' => 'select',
      '#options' => $rt_options,
      '#default_value' => isset($field['settings']['handler_settings']['node_gallery_relationship_type']) ? $field['settings']['handler_settings']['node_gallery_relationship_type'] : NULL,
    );
    /*$bundles = array();
    foreach ($entity_info['bundles'] as $bundle_name => $bundle_info) {
      if (og_is_group_type($entity_type, $bundle_name)) {
        $bundles[$bundle_name] = $bundle_info['label'];
      }
    }

    if (!$bundles) {
      $form['target_bundles'] = array(
        '#type' => 'item',
        '#title' => t('Target bundles'),
        '#markup' => t('Error: The selected "Target type" %entity does not have bundles that are a group type', array('%entity' => $entity_info['label'])),
      );
    }
    else*/ {

      $form['target_bundles'] = array(
        '#type' => 'value',
        '#value' => array(),
      );

      // Get all the other group audience fields in this bundle.
     /* $entity_type = $instance['entity_type'];
      $bundle = $instance['bundle'];
      $fields_info = field_info_fields();
      foreach (field_info_instances($entity_type, $bundle) as $field_name => $field_instance) {
        if ($field_name == $field['field_name']) {
          // This is the current field.
          continue;
        }
        if ($fields_info[$field_name]['type'] != 'entityreference' || $fields_info[$field_name]['settings']['handler'] != 'og') {
          // This is not an Entity reference field.
          continue;
        }

        if (!empty($fields_info[$field_name]['settings']['handler_settings']['primary_field'])) {
          // Field is already a secondary field.
          continue;
        }
        $options[$field_name] = $field_instance['label'] . ' (' . $field_name . ')';
      }*/


     }

    return $form;
  }

/**
   * Build an EntityFieldQuery to get referencable entities.
   */
  protected function buildEntityFieldQuery($match = NULL, $match_operator = 'CONTAINS') {
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', $this->field['settings']['target_type']);
    // Get target bundles from relationship type.
    $relationship_type = node_gallery_api_get_relationship_type(NULL, NULL, $this->field['settings']['handler_settings']['node_gallery_relationship_type']);
    $this->field['settings']['handler_settings']['target_bundles'] = $relationship_type->gallery_types;
    if (!empty($this->field['settings']['handler_settings']['target_bundles'])) {
      $query->entityCondition('bundle', $this->field['settings']['handler_settings']['target_bundles'], 'IN');
    }
    if (isset($match)) {
      $entity_info = entity_get_info($this->field['settings']['target_type']);
      if (isset($entity_info['entity keys']['label'])) {
        $query->propertyCondition($entity_info['entity keys']['label'], $match, $match_operator);
      }
    }

    // Add a generic entity access tag to the query.
    $query->addTag($this->field['settings']['target_type'] . '_access');
    $query->addMetaData('op', 'update');
    $query->addTag('entityreference');
    $query->addMetaData('field', $this->field);
    $query->addMetaData('entityreference_selection_handler', $this);

    // Add the sort option.
    if (!empty($this->field['settings']['handler_settings']['sort'])) {
      $sort_settings = $this->field['settings']['handler_settings']['sort'];
      if ($sort_settings['type'] == 'property') {
        $query->propertyOrderBy($sort_settings['property'], $sort_settings['direction']);
      }
      elseif ($sort_settings['type'] == 'field') {
        list($field, $column) = explode(':', $sort_settings['field'], 2);
        $query->fieldOrderBy($field, $column, $sort_settings['direction']);
      }
    }
    return $query;
  }

}