<?php

/**
 * @file
 * Node gallery theme functions
 */

/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function theme_node_gallery_api_manage_items_form($variables) {
  $form = $variables['form'];
  
  if (empty($form['images'])) {
    return t('This gallery is empty.');
  }
  // Pull in the multi-select functionality from core
  drupal_add_js('misc/tableselect.js');
  // get fieldname to retrieve the filepath for the thumbnail without loading the node
  $relationship_type = node_gallery_api_get_relationship_type($form['#gallery']->type);
  $enable_rotation = FALSE;
  if ($relationship_type->settings['manage_items']['enable_rotation'] && (image_get_toolkit() != 'gd' || function_exists("imagerotate"))) {
    $enable_rotation = TRUE;
    if (module_exists('jquery_ui')) {
      drupal_add_css(drupal_get_path('module', 'jquery_ui') . '/jquery.ui/themes/default/ui.all.css');
      drupal_add_library('system', 'ui.dialog');
      drupal_add_library('system', 'ui.draggable');
      drupal_add_library('system', 'ui.resizable');
    }
    drupal_add_js(drupal_get_path('module', 'node_gallery') . '/js/ng_manage_images.js');
  }
  $field_name = $relationship_type->filefield_name;
  $thumb_imagecache = $form['#thumb_imagecache'];
  $header = array(array(
      'data' => t('Delete'),
      'class' => 'select-all',
    ), t('Preview'), t('Edit'), t('Cover'));
  if ($enable_rotation) {
    $header = array(array(
        'data' => t('Delete'),
        'class' => 'select-all',
      ), t('Preview'), t('Edit'), t('Rotation'), t('Cover'));
  }
  foreach (element_children($form['images']) as $nid) {
    $element = &$form['images'][$nid];
    $file = node_gallery_api_get_item_file($element['edit_form']['nid']['#value'], $field_name);
    $file_view = !empty($file) ? file_view($file, 'node_gallery_file_thumbnail') : '';

    $row = array();
    $row[] = drupal_render($element['remove']);
    // TODO Please change this theme call to use an associative array for the $variables parameter.
    $row[] = drupal_render($file_view);
    //'<span>' . l(/*theme('imagecache', 'node-gallery-admin-thumbnail', */$filepath/*)*/, 'node/' . $nid . '/edit', array('html' => TRUE)) . '</span>';
    $edit = drupal_render($element['edit_form']);
    if (!empty($element['ngid'])) {
      $edit .= drupal_render($element['ngid']);
    }
    $row[] = $edit;
    if ($enable_rotation && !empty($file) && $file->type == 'image') {
      $row[] = drupal_render($element['rotate']);
    }
    else {
      drupal_render($element['rotate']);
      $row[] = '';
    }
    if ($form['is_cover']) {
      $row[] = drupal_render($form['is_cover'][$nid]);
    }
    $rows[] = array('data' => $row);
  }
  $output = theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'upload-attachments')));
  $output .= theme('pager');
  $output .= drupal_render_children($form);
  return $output;
}

/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function theme_node_gallery_content_multiple_values($variables) {
  $element = $variables['element'];
  $nid = $element['#parents'][1];
  $field_name = $element['#field_name'];
  $field = content_fields($field_name);
  $output = '';

  if ($field['multiple'] >= 1) {
    $table_id = $element['#field_name'] . '_values-' . $nid;
    $order_class = $element['#field_name'] . '-delta-order';
    $required = !empty($element['#required']) ? '<span class="form-required" title="' . t('This field is required.') . '">*</span>' : '';

    $header = array(
      array(
        'data' => t('!title: !required', array('!title' => $element['#title'], '!required' => $required)),
        'colspan' => 2,
      ),
      t('Order'),
    );
    $rows = array();

    // Sort items according to '_weight' (needed when the form comes back after
    // preview or failed validation)
    $items = array();
    foreach (element_children($element) as $key) {
      if ($key !== $element['#field_name'] . '_add_more-' . $nid) {
        $items[] = &$element[$key];
      }
    }
    usort($items, '_content_sort_items_value_helper');

    // Add the items as table rows.
    foreach ($items as $key => $item) {
      $item['_weight']['#attributes']['class'] = $order_class;
      $delta_element = drupal_render($item['_weight']);
      $cells = array(
        array(
          'data' => '',
          'class' => 'content-multiple-drag',
        ),
        drupal_render($item),
        array(
          'data' => $delta_element,
          'class' => 'delta-order',
        ),
      );
      $rows[] = array(
        'data' => $cells,
        'class' => 'draggable',
      );
    }

    $output .= theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => $table_id, 'class' => 'content-multiple-table')));
    $output .= $element['#description'] ? '<div class="description">' . $element['#description'] . '</div>' : '';
    $output .= drupal_render($element[$element['#field_name'] . '_add_more-' . $nid]);

    drupal_add_tabledrag($table_id, 'order', 'sibling', $order_class);
  }
  else {
    foreach (element_children($element) as $key) {
      $output .= drupal_render_children($element[$key]);
    }
  }

  return $output;
}

function template_preprocess_node_gallery_api_item_navigator(&$variables) {
  $relationship_type = node_gallery_api_get_relationship_type(NULL, $variables['item']->type);
  $fragment = FALSE;
  if ($relationship_type->settings['item_view']['page_fragment']) {
    $fragment = TRUE;
  }
  $entries = array();
  if (isset($variables['navigator']['prev_nid'])) {
    $entries = array_merge($entries, array('prev', 'first'));
  }
  if (isset($variables['navigator']['next_nid'])) {
    $entries = array_merge($entries, array('next', 'last'));
  }
  foreach ($entries as $type) {
    $nid = $variables['navigator'][$type . '_nid'];
    $variables[$type . '_link'] = $fragment ?
      url('node/' . $nid, array('fragment' => 'node-' . $nid)) :
      url('node/' . $nid);
  }
  if(!empty($variables['navigator']['gallery_nid'])) {
    $variables['gallery_link'] = url('node/' . $variables['navigator']['gallery_nid']);
    $gallery = node_load($variables['navigator']['gallery_nid']);
    $variables['gallery_title'] = check_plain($gallery->title);
  }
}

/**
 * Imports the image properties into the variables namespace for easier theming.
 *
 * @param <type> $variables
 */
function template_preprocess_node_gallery_sort_images_grid_item(&$variables) {
  $variables += $variables['image'];
}

/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function theme_node_gallery_sort_images_form($variables) {
  $form = $variables['form'];
  $output = '';
  if ($form['jquery_ui_integration']['#value']) {
    drupal_add_library('system', 'ui.draggable');
    drupal_add_library('system', 'ui.droppable');
    drupal_add_library('system', 'ui.sortable');
    drupal_add_js(drupal_get_path('module', 'node_gallery') . '/js/ng_sort_jqueryui.js');
    $settings = array('node_gallery' => array(
        'ngid' => $form['ngid']['#value'],
        'url_prefix' => variable_get('clean_url', 0) > 0 ? '' : '?q=',
      ));
    drupal_add_js($settings, array('type' => 'setting', 'scope' => JS_DEFAULT));
    $output .= theme('node_gallery_sort_images_grid', array('images' => $form['#images']));
  }
  else {
    $rows = array();
    $header = array('&uarr;&darr;', t('Preview'), t('Title'), t('Post date'), t('Updated date'), t('Published'), t('Weight'));
    $i = 0;
    foreach ($form['#images'] as $image) {
      $published = ($image['status']) ? t('Yes') : t('No');
      $row = array(
        // The placeholder for the tabledrag icon
        '',
        $image['admin_thumb'],
        $image['title'],
        format_date($image['created'], 'short'),
        format_date($image['changed'], 'short'),
        $published,
        // the weight drop down
        drupal_render($form['images-sort-' . $i]),
      );
      $rows[] = array(
        'data' => $row,
        'class' => 'draggable',
      );
      $i++;
    }
    // this will replace the weight drop downs with drag&drop when javascript is available
    drupal_add_tabledrag('sort-images-table', 'order', 'sibling', 'sort');
    $output .= theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'sort-images-table')));
  }
  // render the left over elements, if any
  $output .= drupal_render_children($form);
  return $output;
}


/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function theme_node_gallery_api_sort_items_form($variables) {
  $form = $variables['form'];
  $output = '';
  /*if ($form['jquery_ui_integration']['#value']) {
    drupal_add_library('system', 'ui.draggable');
    drupal_add_library('system', 'ui.droppable');
    drupal_add_library('system', 'ui.sortable');
    drupal_add_js(drupal_get_path('module', 'node_gallery') . '/js/ng_sort_jqueryui.js');
    $settings = array('node_gallery' => array(
        'ngid' => $form['ngid']['#value'],
        'url_prefix' => variable_get('clean_url', 0) > 0 ? '' : '?q=',
      ));
    drupal_add_js($settings, array('type' => 'setting', 'scope' => JS_DEFAULT));
    $output .= theme('node_gallery_sort_images_grid', array('images' => $form['#images']));
  }
  else {*/
    $rows = array();
    $header = array('&uarr;&darr;', t('Preview'), t('Title'), t('Post date'), t('Updated date'), t('Published'), t('Weight'));
    $i = 0;
    foreach ($form['#images'] as $image) {
      $published = ($image['status']) ? t('Yes') : t('No');
      $row = array(
        // The placeholder for the tabledrag icon
        '',
        drupal_render($image['admin_thumb']),
        $image['title'],
        format_date($image['created'], 'short'),
        format_date($image['changed'], 'short'),
        $published,
        // the weight drop down
        drupal_render($form['images-sort-' . $i]),
      );
      $rows[] = array(
        'data' => $row,
        'class' => array('draggable'),
      );
      $i++;
    }
    // this will replace the weight drop downs with drag&drop when javascript is available
    drupal_add_tabledrag('sort-images-table', 'order', 'sibling', 'sort');
    $output .= theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'sort-images-table')));
  //}
  // render the left over elements, if any
  $output .= drupal_render_children($form);
  return $output;
}



/**
 * Node gallery formatter hooks and callbacks.
 */
function theme_node_gallery_formatter_nextimagelink($element) {
  // TODO: Should this theme node_gallery_formatter_nextimagelink be declared in hook_theme()?
  // Inside a view $element may contain null data. In that case, just return.
  if (empty($element['#item']['fid'])) {
    return '';
  }

  // If node type is not in a gallery relationship or gid is not set, just render the default formatter.
  $imagetag = theme_imagecache_formatter_default($element);
  $node = $element['#node'];
  if (!in_array($node->type, (array) node_gallery_get_types('item'))
           || !isset($node->ngid)) {
    return $imagetag;
  }
  $relationship = node_gallery_get_relationship(NULL, $node->type);
  $next = node_gallery_get_next_image($node->ngid, $node->nid);
  if ($next !== NULL) {
    $class = 'imagefield image-nextimagelink imagefield-' . $element['#field_name'];
    $options = array(
      'attributes' => array('class' => $class),
      'html' => TRUE,
    );
    if ($relationship['settings']['node_images_page_fragment']) {
      $options['fragment'] = 'node-' . $next;
    }
    return l($imagetag, 'node/' . $next, $options);
  }
  else {
    // no link on the last image
    return $imagetag;
  }
}

/**
 * Link an image to its gallery.
 */
function theme_node_gallery_formatter_gallerylink($element) {
  // TODO: Should this theme node_gallery_formatter_gallerylink be declared in hook_theme()?
  if (empty($element['#item']['fid'])) {
    return '';
  }
  $imagetag = theme_imagecache_formatter_default($element);
  if (!in_array($element['#node']->type, (array) node_gallery_get_types('item'))) {
    return $imagetag;
  }
  $node = node_load($element['#node']->nid);
  $class = 'imagefield image-gallerylink imagefield-' . $element['#field_name'];
  $options = array(
    'attributes' => array('class' => $class),
    'html' => TRUE,
  );
  return l($imagetag, 'node/' . $node->ngid, $options);
}
